/*
 * Project name:
     Slave;
 * Copyright:
     (c) MikroElektronika, 2005-2010.
  * Revision History:
     20101209:
       - initial release (SZ);
 * Description:
     This is a simple project which demonstrates the use of Easy BlueTooth module.
     After establishing the connection Master repeatedly sends two strings ("mikroE" and "Easy BlueTooth")
     to the Slave. The Slave receives them and displays them at the 2x16 LCD.
 * Test configuration:
     MCU:             ATmega16
                      http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf
     Dev.Board:       EasyAVR6
                      http://www.mikroe.com/eng/products/view/321/easyavr6-development-system/
     Oscillator:      External, 8.0000 MHz
     Ext. Modules:    Easy BlueTooth Board at PORTD
                      ac:EasyBT
                      http://www.mikroe.com/eng/products/view/621/easybluetooth-board/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/eng/products/view/228/mikroc-pro-for-avr/
 * NOTES:
  - Easy BlueTooth board: Turn ON SW1.3 and SW1.6. and SUPPLY SELECT jumper - 5V.
  - Be sure that (near by) no BlueTooth devices other than Master are present.
  - Turn on both modules at the same time and wait for about one minute
    until the connection between the Master and the Slave establishes.
  - Turn on switches SW6.1, SW6.5, SW6.7, SW7.1 and SW7.5.
  - Turn on COG LCD 2x16 switches SW10.1, SW10.2, SW10.3, SW10.4, SW10.5 and SW10.6.
*/

// Port Expander module connections
sbit  SPExpanderRST at PORTB2_bit;
sbit  SPExpanderCS  at PORTB1_bit;
sbit  SPExpanderRST_Direction at DDB2_bit;
sbit  SPExpanderCS_Direction  at DDB1_bit;
// End Port Expander module connections

// responses to parse
const BT_CMD  = 1;
const BT_AOK  = 2;
const BT_CONN = 3;
const BT_END  = 4;

char txt[15];
unsigned short i, tmp, DataReady;
char CMD_mode = 1;

char BT_state = 0;
char response_rcvd = 0;
char responseID = 0, response = 0;

// Uart Rx interrupt handler
void interrupt() org IVT_ADDR_USART_RXC {
    tmp = UART1_Read();                         // Get received byte

  if (CMD_mode){

    /* The responses expected from the EasyBT module:
    CMD
    AOK
    AOK
    AOK
    AOK
    AOK
    END
    SlaveCONNECTmikroE
    EasyBlueTooth
    mikroE ...
    EasyBlueTooth*/


// Process reception through state machine
// We are parsing CMD<cr><lf>, AOK<cr><lf>, CONN<cr> and END<cr><lf> responses
    switch (BT_state) {
      case  0: {
                response = 0;                   // Clear response
                if (tmp == 'C')                 // We have 'C', it could be CMD<cr><lf>  or CONN
                  BT_state = 1;                 // Expecting 'M' or 'N'
                if (tmp == 'A')                 // We have 'A', it could be AOK<cr><lf>
                  BT_state = 11;                // expecting 'O'
                if (tmp == 'E')                 // We have 'E', it could be END<cr><lf>
                  BT_state = 31;                // expecting 'N'
                break;                          // ...
               }

      case  1: {
                if (tmp == 'M')
                  BT_state = 2;
                else if (tmp == 'O')
                  BT_state = 22;
                else
                  BT_state = 0;
                break;
               }


      case  2: {
                 if (tmp == 'D') {
                  response = BT_CMD;            // CMD
                  BT_state = 40;
                  }
                  else
                  BT_state = 0;
                  break;
               }


      case 11: {
                if (tmp == 'O')
                  BT_state = 12;
                else
                  BT_state = 0;
                break;
               }

      case 12: {
                if (tmp == 'K'){
                   response = BT_AOK;           // AOK
                   BT_state = 40;
                }else
                   BT_state = 0;
                break;
               }


      case 22: {
                if (tmp == 'N')
                 BT_state = 23;
                else
                  BT_state = 0;
                break;
               }

     case 23: {
                 if (tmp == 'N') {
                  response = BT_CONN;           // SlaveCONNECTmikroE
                  response_rcvd = 1;
                  responseID = response;
                  }
                  BT_state = 0;
                  break;
               }

     case 31: {
                if (tmp == 'N')
                  BT_state = 32;
                else
                  BT_state = 0;
                 break;
               }

      case 32: {
                 if (tmp == 'D') {
                  response = BT_END;            // END
                  BT_state = 40;
                  }
                  else
                  BT_state = 0;
                  break;
               }

      case 40: {
                if (tmp == 13)
                  BT_state = 41;
                else
                  BT_state = 0;
                break;
               }


      case 41: {
                if (tmp == 10){
                response_rcvd = 1;
                responseID = response;
                }
                BT_state = 0;
                break;
                }

      default: {
                BT_state = 0;
                break;
               }
    }
  }
  else {
       if (tmp == 13) {
         txt[i] = 0;                            // Puting 0 at the end of the string
         DataReady = 1;                         // Data is received
       }
        else {
          txt[i] = tmp;                         // Moving the data received from UART to string txt[]
          i++;
     }
   }
}


// Get BlueTooth response, if there is any
char BT_Get_Response() {
  if (response_rcvd) {
    response_rcvd = 0;
    return responseID;
  }
  else
    return 0;
}


void main() {
// Set RTS pin to zero, we will use only RX i TX
  DDD3_bit = 0;
  PORTD3_bit = 0;

  Delay_ms(500);
  UART1_init(9600);                             // Initialize UART1 module
  Delay_ms(1000);

  SREG_I_bit  = 1;                              // Enable global interrupt
  RXCIE_bit   = 1;                              // Enable interrupt on UART receive

  // Port Expander Library uses SPI1 module
  SPI1_Init();                                  // Initialize SPI module used with PortExpander
  SPI_Lcd_Config(0);                            // Initialize Lcd over SPI interface
  SPI_Lcd_Cmd(_LCD_CLEAR);                      // Clear display
  SPI_Lcd_Cmd(_LCD_CURSOR_OFF);                 // Turn cursor off

  SPI_Lcd_Out(1,1,"Connecting!");
  SPI_Lcd_Out(2,1,"Please, wait...");


  do {
      UART1_Write_Text("$$$");                  // Enter command mode
      Delay_ms(500);
  } while (BT_Get_Response() != BT_CMD);

  do {
      UART1_Write_Text("SN,Bluetooth-Slave");   // Name of device
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

   do {
      UART1_Write_Text("SO,Slave");             // Extended status string
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART1_Write_Text("SM,0");                 // Set mode (0 = slave, 1 = master, 2 = trigger, 3 = auto, 4 = DTR, 5 = ANY)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART1_Write_Text("SA,1");                 // Authentication (1 to enable, 0 to disable)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
     UART1_Write_Text("SP,mikroe");             // Security pin code (mikroe)
      UART1_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
     UART1_Write_Text("---");                   // Security pin code (mikroe)
     UART1_Write(13);                           // CR
     Delay_ms(500);
  } while (BT_Get_Response() != BT_END);

  while (BT_Get_Response() != BT_CONN);

  SPI_LCD_Cmd(_LCD_CLEAR);

  CMD_mode = 0;

  DataReady = 0;

  SPI_Lcd_Out(1,1,"Connected!");
  Delay_ms(1000);
  SPI_LCD_Cmd(_LCD_CLEAR);                      // Clear display
  SPI_Lcd_Out(1,1,"Receiving...");

  while (1) {
    tmp = 0;                                    // Init variables
    i = 0;
    memset(txt, 0, 15);                         // Clear array of chars

    RXCIE_bit   = 1;                            // Interrupts allowed

    while (!DataReady) ;                        // Wait while the data is received

    RXCIE_bit   = 0;                            // Interrupts forbiden
    DataReady = 0;

    SPI_LCD_Cmd(_LCD_CLEAR);                    // Clear display
    SPI_Lcd_Out(1,1,"Received:");
    SPI_Lcd_Cmd(_LCD_SECOND_ROW);
    i = 0;
    while (txt[i] != 0){

       SPI_Lcd_Chr_CP(txt[i]);                  // Displaying the received text on the LCD
       i++;                                     // There is delay before the data is displayed
    }                                           // This is due the init duration and synchronization of two Zigbit modules

  }
}