/*
 * Project name:
     Slave;
 * Copyright:
     (c) MikroElektronika, 2005-2010.
  * Revision History:
     20101209:
       - initial release (SZ);
 * Description:
     This is a simple project which demonstrates the use of Easy BlueTooth module.
     After establishing the connection Master repeatedly sends two strings ("mikroE" and "Easy BlueTooth")
     to the Slave. The Slave receives them and displays them at the 2x16 LCD.
 * Test configuration:
     MCU:             PIC24FJ96GA010
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39747e.pdf
     Dev.Board:       LV24-33 v6
                      http://www.mikroe.com/eng/products/view/430/lv-24-33-v6-development-system/
     Oscillator:      HS-PLL, 8.00000 MHz
     Ext. Modules:    Easy BlueTooth Board at PORTF
                      ac:EasyBT
                      http://www.mikroe.com/eng/products/view/621/easybluetooth-board/
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/231/mikroc-pro-for-dspic30-33-and-pic24/
 * NOTES:
  - Easy BlueTooth board: Turn ON SW1.2 and SW1.5. and SUPPLY SELECT jumper - 3.3V.
  - Be sure that (near by) no BlueTooth devices other than Master are present.
  - Turn on both modules at the same time and wait for about one minute
    until the connection between the Master and the Slave establishes.
*/

// responses to parse
const BT_CMD  = 1;
const BT_AOK  = 2;
const BT_CONN = 3;
const BT_END  = 4;

// LCD module connections
sbit LCD_RS at LATB2_bit;
sbit LCD_EN at LATB3_bit;
sbit LCD_D4 at LATB4_bit;
sbit LCD_D5 at LATB5_bit;
sbit LCD_D6 at LATB6_bit;
sbit LCD_D7 at LATB7_bit;

sbit LCD_RS_Direction at TRISB2_bit;
sbit LCD_EN_Direction at TRISB3_bit;
sbit LCD_D4_Direction at TRISB4_bit;
sbit LCD_D5_Direction at TRISB5_bit;
sbit LCD_D6_Direction at TRISB6_bit;
sbit LCD_D7_Direction at TRISB7_bit;
// End LCD module connections

char txt[15];
unsigned short i, tmp, DataReady;
char CMD_mode = 1;

char BT_state = 0;
char response_rcvd = 0;
char responseID = 0, response = 0;

// Uart Rx interrupt handler
void interrupt() iv IVT_ADDR_U2RXINTERRUPT {
    tmp = UART2_Read();                         // Get received byte
    U2RXIF_bit = 0;

  if (CMD_mode){

    /* The responses expected from the EasyBT module:
    CMD
    AOK
    AOK
    AOK
    AOK
    AOK
    END
    SlaveCONNECTmikroE
    EasyBlueTooth
    mikroE ...
    EasyBlueTooth*/


// Process reception through state machine
// We are parsing CMD<cr><lf>, AOK<cr><lf>, CONN<cr> and END<cr><lf> responses
    switch (BT_state) {
      case  0: {
                response = 0;                   // Clear response
                if (tmp == 'C')                 // We have 'C', it could be CMD<cr><lf>  or CONN
                  BT_state = 1;                 // Expecting 'M' or 'N'
                if (tmp == 'A')                 // We have 'A', it could be AOK<cr><lf>
                  BT_state = 11;                // expecting 'O'
                if (tmp == 'E')                 // We have 'E', it could be END<cr><lf>
                  BT_state = 31;                // expecting 'N'
                break;                          // ...
               }

      case  1: {
                if (tmp == 'M')
                  BT_state = 2;
                else if (tmp == 'O')
                  BT_state = 22;
                else
                  BT_state = 0;
                break;
               }


      case  2: {
                 if (tmp == 'D') {
                  response = BT_CMD;            // CMD
                  BT_state = 40;
                  }
                  else
                  BT_state = 0;
                  break;
               }


      case 11: {
                if (tmp == 'O')
                  BT_state = 12;
                else
                  BT_state = 0;
                break;
               }

      case 12: {
                if (tmp == 'K'){
                   response = BT_AOK;           // AOK
                   BT_state = 40;
                }else
                   BT_state = 0;
                break;
               }


      case 22: {
                if (tmp == 'N')
                 BT_state = 23;
                else
                  BT_state = 0;
                break;
               }

     case 23: {
                 if (tmp == 'N') {
                  response = BT_CONN;           // SlaveCONNECTmikroE
                  response_rcvd = 1;
                  responseID = response;
                  }
                  BT_state = 0;
                  break;
               }

     case 31: {
                if (tmp == 'N')
                  BT_state = 32;
                else
                  BT_state = 0;
                 break;
               }

      case 32: {
                 if (tmp == 'D') {
                  response = BT_END;            // END
                  BT_state = 40;
                  }
                  else
                  BT_state = 0;
                  break;
               }

      case 40: {
                if (tmp == 13)
                  BT_state = 41;
                else
                  BT_state = 0;
                break;
               }


      case 41: {
                if (tmp == 10){
                response_rcvd = 1;
                responseID = response;
                }
                BT_state = 0;
                break;
                }

      default: {
                BT_state = 0;
                break;
               }
    }
  }
  else {
       if (tmp == 13) {
         txt[i] = 0;                            // Puting 0 at the end of the string
         DataReady = 1;                         // Data is received
       }
        else {
          txt[i] = tmp;                         // Moving the data received from UART to string txt[]
          i++;
     }
   }
}


// Get BlueTooth response, if there is any
char BT_Get_Response() {
  if (response_rcvd) {
    response_rcvd = 0;
    return responseID;
  }
  else
    return 0;
}


void main() {
 // All pins as digital I/Os
  AD1PCFG = 0xFFFF;                             // Configure AN pins as digital I/O
// Set RTS pin to zero, we will use only RX i TX
  TRISF3_bit = 0;
  LATC3_bit = 0;

// Enable uart rx interrupt
  NSTDIS_bit = 1;                               // No nesting of interrupts
  U2RXIF_bit = 0;                               // Ensure interrupt not pending
  U2RXIE_bit = 1;                               // Enable intterupt

  LCD_Init();                                   // LCD Init
  LCD_Cmd(_LCD_CLEAR);
  Lcd_Cmd(_LCD_CURSOR_OFF);

  Lcd_Out(1,1,"Connecting!");
  Lcd_Out(2,1,"Please, wait...");

  UART2_init(9600);                             // Initialize UART2 module
  Delay_ms(1500);


  do {
      UART2_Write_Text("$$$");                  // Enter command mode
      Delay_ms(500);
  } while (BT_Get_Response() != BT_CMD);

  do {
      UART2_Write_Text("SN,Bluetooth-Slave");   // Name of device
      UART2_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

   do {
      UART2_Write_Text("SO,Slave");             // Extended status string
      UART2_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART2_Write_Text("SM,0");                 // Set mode (0 = slave, 1 = master, 2 = trigger, 3 = auto, 4 = DTR, 5 = ANY)
      UART2_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
      UART2_Write_Text("SA,1");                 // Authentication (1 to enable, 0 to disable)
      UART2_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
     UART2_Write_Text("SP,mikroe");             // Security pin code (mikroe)
      UART2_Write(13);                          // CR
      Delay_ms(500);
  } while (BT_Get_Response() != BT_AOK);

  do {
     UART2_Write_Text("---");                   // Security pin code (mikroe)
     UART2_Write(13);                           // CR
     Delay_ms(500);
  } while (BT_Get_Response() != BT_END);

  while (BT_Get_Response() != BT_CONN);

  LCD_Cmd(_LCD_CLEAR);

  CMD_mode = 0;

  DataReady = 0;

  Lcd_Out(1,1,"Connected!");
  Delay_ms(1000);
  LCD_Cmd(_LCD_CLEAR);                          // Clear display
  Lcd_Out(1,1,"Receiving...");

  while (1) {
    tmp = 0;                                    // Init variables
    i = 0;
    memset(txt, 0, 15);                         // Clear array of chars

    U2RXIE_bit = 1;                             // Interrupts allowed

    while (!DataReady) ;                        // Wait while the data is received

    U2RXIE_bit = 0;                             // Interrupts forbiden
    DataReady = 0;

    LCD_Cmd(_LCD_CLEAR);                        // Clear display
    Lcd_Out(1,1,"Received:");
    Lcd_Cmd(_LCD_SECOND_ROW);
    i = 0;
    while (txt[i] != 0){

       Lcd_Chr_CP(txt[i]);                      // Displaying the received text on the LCD
       i++;                                     // There is delay before the data is displayed
    }                                           // This is due the init duration and synchronization of two Zigbit modules

  }
}